﻿; Module/File:     Menu_AddStockImageByLabel.pb
; Function:        Add stock images to menu items by title and label - Linux gtk2/gtk3
; Author:          Omi
; Date:            Nov. 30, 2015
; Version:         0.2
; Target Compiler: PureBasic 5.22/5.31/5.40
; Target OS:       Linux: (X/K/L)ubuntu, Mint, 32/64, Ascii/Uni
;--------------------------------------------------------------
;works on Menu and ImageMenu

EnableExplicit

ImportC ""
	gtk_image_new_from_icon_name(stock_id.p-utf8, size)
	gtk_container_get_children(*container.GtkContainer)
	gtk_container_remove(*container.GtkContainer, *widget.GtkWidget)
	gtk_menu_item_get_label(*menu_item.GtkMenuItem)
	gtk_menu_item_get_submenu(*menu_item.GtkMenuItem)
	gtk_menu_get_attach_widget(*menu.GtkMenu)
	gtk_image_menu_item_set_image(*image_menu_item, *image)
	gtk_widget_get_name(*widget.GtkWidget)
EndImport


; Object constants
#Win_Main = 0


Enumeration;                           Menu
	#Mn1
	#Mn1_Item11
	#Mn1_Item12
	#Mn1_Item13
	#Mn1_Item14
	#Mn1_Item141
	#Mn1_Item142
	#Mn1_Item15
	#Mn1_Item21
	#Mn1_Item22
	#Mn1_Item23
	#Mn1_Item24
EndEnumeration

Global.i gEvent, gQuit
Global.s gGtkSubSytem


Procedure.s GetGtkSubSytem()
	CompilerIf #PB_Compiler_Version < 540
		CompilerIf Subsystem("gtk3")
			gGtkSubSytem= "gtk3"
		CompilerElse
			gGtkSubSytem= "gtk2"
		CompilerEndIf
	CompilerElse
		CompilerIf Subsystem("gtk2")
			gGtkSubSytem= "gtk2"
		CompilerElse
			gGtkSubSytem= "gtk3"
		CompilerEndIf
	CompilerEndIf
	ProcedureReturn gGtkSubSytem
EndProcedure

;-
;- RecentFile-Library (until PB returns the widgets-adr ;-)_____________

Procedure.s Menu_GetItemLabel(*widget.GtkWidget);                                        works for popups, menus, submenus
	Protected   *subType.GtkImageMenuItem
	Protected.s ItemType, ItemLabel
	
	If *widget
		ItemType= PeekS(gtk_widget_get_name_(*widget), -1, #PB_UTF8);                         Menu item type
		If gGtkSubSytem= "gtk3"
			If ItemType <> "GtkSeparatorMenuItem";                                             else separator is harmed
				ItemLabel= PeekS(gtk_menu_item_get_label(*widget), -1, #PB_UTF8);                Label always in item
			EndIf
		Else;                                                                                gtk2
			*subType= *widget
			If *subType\menu_item\item\bin\child
				If PeekS(gtk_widget_get_name_(*subType\menu_item\item\bin\child), -1, #PB_UTF8) = "GtkAccelLabel";  gtk2 uses subitems for labels
					ItemLabel= PeekS(gtk_label_get_text_(*subType\menu_item\item\bin\child), -1, #PB_UTF8)
				EndIf
			EndIf
		EndIf
		ItemLabel= RemoveString(ItemLabel, "_", #PB_String_CaseSensitive, 1, 1);             "_" remove
	EndIf
	
	ProcedureReturn ItemLabel
EndProcedure

Procedure.i Menu_AddStockImageByLabel(*menu, Menu, Title.s, sItemLabel.s, StockItem.s)
	Protected   *widget.GtkWidget, *SubMenu.GtkWidget
	Protected   *list.GList= gtk_container_get_children(*menu)
	Protected   *Agree= #Null
	Protected.s ItemLabel
	Protected.i I
	
	For I= 0 To g_list_length_(*list)- 1
		*widget= g_list_nth_data_(*list, I)
		
		If *widget
			*SubMenu= gtk_menu_item_get_submenu(*widget)
			ItemLabel= Menu_GetItemLabel(*widget)
			If sItemLabel = ItemLabel
				If gtk_menu_get_attach_widget(*menu)
					If Menu_GetItemLabel(gtk_menu_get_attach_widget(*menu)) = Title
						*Agree= *widget
					EndIf
				Else
					If Title = ""
						*Agree= *widget
					EndIf
				EndIf
				If *Agree
					Protected *image.GtkImage= gtk_image_new_from_icon_name(StockItem, #GTK_ICON_SIZE_MENU)
					gtk_image_menu_item_set_image(*Agree, *image)
					Break
				EndIf
			EndIf
			
			If *SubMenu
				*Agree= Menu_AddStockImageByLabel(*SubMenu, Menu, Title, sItemLabel, StockItem)
				If *Agree : Break : EndIf
			EndIf
		EndIf
	Next I
	
	g_list_free_(*list)
	ProcedureReturn *Agree
EndProcedure

;-----------------------

Procedure CreateWindow_Main()
	OpenWindow(#Win_Main, 0, 0, 400, 300, "MenuItems w. Stock-Images", #PB_Window_SystemMenu | #PB_Window_ScreenCentered)
EndProcedure

Procedure Create_Mn1()
		CreateImageMenu(#Mn1, WindowID(#Win_Main))
		MenuTitle("File")
		MenuItem(#Mn1_Item11, "&New"        + #TAB$ + "Ctrl+N")
		MenuItem(#Mn1_Item12, "&Open"       + #TAB$ + "Ctrl+O")
		MenuItem(#Mn1_Item13, "&Save"       + #TAB$ + "Ctrl+S")
		MenuBar()
		MenuItem(#Mn1_Item14, "&Print"      + #TAB$ + "Ctrl+P")
		MenuBar()
		
		OpenSubMenu("Recent files")
			MenuItem(#Mn1_Item141, "SubItem 1")
			MenuItem(#Mn1_Item142, "SubItem 2")
		CloseSubMenu()
		MenuBar()
		
		MenuItem(#Mn1_Item15, "Quit")
		
		MenuTitle("Edit")
		MenuItem(#Mn1_Item21, "&Copy"       + #TAB$ + "Ctrl+C")
		MenuItem(#Mn1_Item22, "Cut"         + #TAB$ + "Ctrl+X")
		MenuItem(#Mn1_Item23, "Paste"       + #TAB$ + "Ctrl+V")
		MenuBar()
		MenuItem(#Mn1_Item24, "Select &all" + #TAB$ + "Ctrl+A")
		
		DisableMenuItem(#Mn1, #Mn1_Item14, #True)
EndProcedure

gGtkSubSytem= GetGtkSubSytem()
CreateWindow_Main()
Create_Mn1()

Menu_AddStockImageByLabel(MenuID(#Mn1), #Mn1, "File", "New",        "gtk-new")
Menu_AddStockImageByLabel(MenuID(#Mn1), #Mn1, "File", "Open",       "gtk-open")
Menu_AddStockImageByLabel(MenuID(#Mn1), #Mn1, "File", "Save",       "gtk-save")
Menu_AddStockImageByLabel(MenuID(#Mn1), #Mn1, "File", "Print",      "gtk-print")
Menu_AddStockImageByLabel(MenuID(#Mn1), #Mn1, "File", "Quit",       "gtk-quit")

Menu_AddStockImageByLabel(MenuID(#Mn1), #Mn1, "Edit", "Copy",       "gtk-copy")
Menu_AddStockImageByLabel(MenuID(#Mn1), #Mn1, "Edit", "Cut",        "gtk-cut")
Menu_AddStockImageByLabel(MenuID(#Mn1), #Mn1, "Edit", "Paste",      "gtk-paste")
Menu_AddStockImageByLabel(MenuID(#Mn1), #Mn1, "Edit", "Select all", "gtk-select-all")

Repeat
	gEvent= WaitWindowEvent()
	
	Select gEvent
			
		Case #PB_Event_CloseWindow
			gQuit= #True
			
		Case #PB_Event_Menu
			Debug GetMenuItemText(#Mn1, EventMenu())
			
	EndSelect
	
Until gQuit
; IDE Options = PureBasic 5.42 LTS (Linux - x86)
; CursorPosition = 97
; FirstLine = 97
; Folding = --
; EnableUnicode
; EnableXP