﻿; Module/File:     Popup_ItemDelByLabel.pb
; Function:        Removes popup item by subtitle and item label - Linux gtk2/gtk3
; Author:          Omi
; Date:            Nov. 04, 2015
; Version:         0.1
; Target Compiler: PureBasic 5.22/5.31/5.40
; Target OS:       Linux: (X/K/L)ubuntu, Mint, 32/64, Ascii/Uni
;--------------------------------------------------------------
;look @ similar 'Hide'-function (could be more digestible): Popup_ItemHideByLabel.pb

EnableExplicit

ImportC ""
	gtk_container_get_children(*container.GtkContainer)
	gtk_container_remove(*container.GtkContainer, *widget.GtkWidget)
	gtk_menu_item_get_label(*menu_item.GtkMenuItem)
	gtk_menu_item_get_submenu(*menu_item.GtkMenuItem)
	gtk_menu_get_attach_widget(*menu.GtkMenu)
	gtk_widget_get_name(*widget.GtkWidget)
EndImport


; Object constants
#Win_Main = 0

#But1     = 0
#But2     = 1
#But3     = 2


Enumeration;                           Popup
	#Pu1
	#Pu1_Item1
	#Pu1_Item2
	#Pu1_Item3
	#Pu1_Item4
	#Pu1_Item51
	#Pu1_Item52
	#Pu1_Item53
	#Pu1_Item6
EndEnumeration

#G_TYPE_STRING= 64

Global.i gEvent, gQuit
Global.s gGtkSubSytem


Procedure.s GetGtkSubSytem()
	CompilerIf #PB_Compiler_Version < 540
		CompilerIf Subsystem("gtk3")
			gGtkSubSytem= "gtk3"
		CompilerElse
			gGtkSubSytem= "gtk2"
		CompilerEndIf
	CompilerElse
		CompilerIf Subsystem("gtk2")
			gGtkSubSytem= "gtk2"
		CompilerElse
			gGtkSubSytem= "gtk3"
		CompilerEndIf
	CompilerEndIf
	ProcedureReturn gGtkSubSytem
EndProcedure

;-
;- RecentFile-Library (until PB returns the widgets-adr ;-)_____________

Procedure.s Menu_GetItemLabel(*widget.GtkWidget);                                        works for popups, menus, submenus
	Protected   *subType.GtkImageMenuItem
	Protected.s ItemType, ItemLabel
	
	If *widget
		ItemType= PeekS(gtk_widget_get_name_(*widget), -1, #PB_UTF8);                        Menu item type
		If gGtkSubSytem= "gtk3"
			If ItemType <> "GtkSeparatorMenuItem";                                             else separator is harmed
				ItemLabel= PeekS(gtk_menu_item_get_label(*widget), -1, #PB_UTF8);                Label always in item
			EndIf
		Else;                                                                                gtk2
			*subType= *widget
			If *subType\menu_item\item\bin\child
				If PeekS(gtk_widget_get_name_(*subType\menu_item\item\bin\child), -1, #PB_UTF8) = "GtkAccelLabel";  gtk2 uses subitems for labels
					ItemLabel= PeekS(gtk_label_get_text_(*subType\menu_item\item\bin\child), -1, #PB_UTF8)
				EndIf
			EndIf
		EndIf
		ItemLabel= RemoveString(ItemLabel, "_", #PB_String_CaseSensitive, 1, 1);             remove "_" 
	EndIf
	
	ProcedureReturn ItemLabel
EndProcedure

Procedure.i Popup_RemoveItemByLabel(*menu, Menu, Title.s, sItemLabel.s)
	Protected   *widget.GtkWidget, *SubMenu.GtkWidget
	Protected   *list.GList= gtk_container_get_children(*menu)
	Protected   *Removed= #Null
	Protected.s ItemLabel
	Protected.i I
	
	For I= 0 To g_list_length_(*list)- 1
		*widget= g_list_nth_data_(*list, I)
		
		If *widget
			*SubMenu= gtk_menu_item_get_submenu(*widget)
			ItemLabel= Menu_GetItemLabel(*widget)
			If sItemLabel = ItemLabel
				If gtk_menu_get_attach_widget(*menu)
					If Menu_GetItemLabel(gtk_menu_get_attach_widget(*menu)) = Title
						*Removed= *widget
					EndIf
				Else
					If Title = ""
						*Removed= *widget
					EndIf
				EndIf
				If *Removed
					gtk_container_remove(*menu, *widget)
					Break
				EndIf
			EndIf
			
			If *SubMenu
				*Removed= Popup_RemoveItemByLabel(*SubMenu, Menu, Title, sItemLabel)
				If *Removed : Break : EndIf
			EndIf
		EndIf
	Next I
	
	g_list_free_(*list)
	ProcedureReturn *Removed
EndProcedure

;-----------------------

Procedure CreateWindow_Main()
	OpenWindow(#Win_Main, 0, 0, 400, 300, "Remove PopupItem by labels", #PB_Window_SystemMenu | #PB_Window_ScreenCentered)
	ButtonGadget(#But1,   5,  5, 390, 26, "Open Popup ...")
	ButtonGadget(#But2,  10, 50, 380, 26, "Remove Popup Item 'Print'")
	ButtonGadget(#But3,  10, 80, 380, 26, "Remove Popup Item 'Format / UTF8'")
EndProcedure

Procedure Create_Pu1()
	CreatePopupMenu(#Pu1)
	MenuItem(#Pu1_Item1, "New")
	MenuItem(#Pu1_Item2, "Open")
	MenuItem(#Pu1_Item3, "Save")
	MenuItem(#Pu1_Item4, "Print")
	MenuBar()
	
	OpenSubMenu("Format")
	MenuItem(#Pu1_Item51, "Ascii")
	MenuItem(#Pu1_Item52, "UTF8")
	MenuItem(#Pu1_Item53, "Unicode")
	CloseSubMenu()
	MenuBar()
	
	MenuItem(#Pu1_Item6, "Quit")
EndProcedure

gGtkSubSytem= GetGtkSubSytem()
CreateWindow_Main()
Create_Pu1()

Repeat
	gEvent= WaitWindowEvent()
	
	Select gEvent
			
		Case #PB_Event_CloseWindow
			gQuit= #True
			
		Case #PB_Event_Gadget
			
			Select EventGadget()
					
				Case #But1
					DisplayPopupMenu(#Pu1, WindowID(#Win_Main))
					
				Case #But2
					Popup_RemoveItemByLabel(MenuID(#Pu1), #Pu1, "", "Print");            no title, ItemID for "Print"
					
				Case #But3
					Popup_RemoveItemByLabel(MenuID(#Pu1), #Pu1, "Format", "UTF8");       subtitle "Format", ItemID for "Format / UTF8"
					
			EndSelect
			
		Case #PB_Event_Menu
			Debug GetMenuItemText(#Pu1, EventMenu())
			
	EndSelect
	
Until gQuit
; IDE Options = PureBasic 5.40 LTS (Linux - x64)
; CursorPosition = 9
; Folding = --
; EnableUnicode
; EnableXP