﻿; Module/File:     Desktop_Notify.pb
; Function:        Shows a desktop-notify (Benachrichtigung) - Linux
; Author:          remi_meier, Omi
; Date:            Mar. 20, 2015
; Version:         0.1
; Target Compiler: PureBasic 5.22/5.31
; Target OS:       Linux: (X/K/L)ubuntu, Mint, 32/64, Ascii/Uni
;--------------------------------------------------------------

EnableExplicit

;***********
CompilerIf #PB_Compiler_Processor= #PB_Processor_x64
	ImportC "/usr/lib/x86_64-linux-gnu/libnotify.so.4"; "/usr/lib/libnotify.so.4"
CompilerElseIf #PB_Compiler_Processor= #PB_Processor_x86
	ImportC "/usr/lib/i386-linux-gnu/libnotify.so.4"; "/usr/lib/libnotify.so.4"
CompilerEndIf
	notify_notification_new.i(summary.p-utf8, body.p-utf8, icon.p-utf8)
	notify_notification_add_action(*notification, action.p-utf8, label.p-utf8, *callback, *user_data, *free_func)
	notify_notification_clear_actions(*notification)
	notify_notification_clear_hints(*notification)
	notify_notification_close.l(*notification, _gerror.i=0)
	notify_notification_get_closed_reason.l(*notification)
	notify_notification_set_app_name(*notification, app_name.p-utf8)
	notify_notification_set_category(*notification, category.p-utf8)
	notify_notification_set_hint(*notification, key.p-utf8, *value)
	notify_notification_set_hint_byte(*notification, key.p-utf8, value.a)
	notify_notification_set_hint_byte_array(*notification, key.p-utf8, value.a, len.q)
	notify_notification_set_hint_double(*notification, key.p-utf8, value.d)
	notify_notification_set_hint_int32(*notification, key.p-utf8, value.l)
	notify_notification_set_hint_string(*notification, key.p-utf8, value.p-utf8)
	notify_notification_set_hint_uint32(*notification, key.p-utf8, value.l)
	notify_notification_set_icon_from_pixbuf(*notification, *icon)
	notify_notification_set_image_from_pixbuf(*notification, *pixbuf)
	notify_notification_set_timeout(*notification, timeout.l)
	notify_notification_set_urgency(*notification, urgency.l)
	notify_notification_show.l(*notification, _gerror.i=0)
	notify_notification_update.l(*notification, summary.p-utf8, body.p-utf8, icon.p-utf8)
	notify_get_app_name.i()
	notify_get_server_caps.i()
	notify_get_server_info.l(ret_name.p-utf8, ret_vendor.p-utf8, ret_version.p-utf8, ret_spec_version.p-utf8)
	notify_init.l(app_name.p-utf8)
	notify_is_initted.l()
	notify_set_app_name(app_name.p-utf8)
	notify_uninit()
EndImport

;- NAMESPACE: Notify from "libnotify.so.4"
#NOTIFY_EXPIRES_NEVER = 0
#NOTIFY_VERSION_MAJOR = 0
#NOTIFY_VERSION_MICRO = 3
#NOTIFY_VERSION_MINOR = 7

Enumeration
	#NOTIFY_URGENCY_LOW
	#NOTIFY_URGENCY_NORMAL
	#NOTIFY_URGENCY_CRITICAL
EndEnumeration

CompilerIf Defined(NotifyNotificationClass, #PB_Structure)
CompilerElse
	Structure NotifyNotificationClass
		*parent_class
		*closed
	EndStructure
CompilerEndIf

CompilerIf Defined(NotifyNotificationPrivate, #PB_Structure)
CompilerElse
	Structure NotifyNotificationPrivate
	EndStructure
CompilerEndIf

;***********

; Object constants
#MainWin  = 0
#But1     = 0
#But2     = 1

#WinX=300
#WinY=200
#WinW=250
#WinH=200

Global.i gEvent, gQuit
Global gNotify

If OpenWindow(#MainWin, #WinX, #WinY, #WinW, #WinH, "Desktop notify", #PB_Window_SystemMenu | #PB_Window_ScreenCentered)
	ButtonGadget(#But1, 5, 5, 240, 26, "Show a desktop-notify")
	ButtonGadget(#But2, 5, 35, 240, 26, "Show a urgent desktop-notify")
	
	Repeat
		gEvent= WaitWindowEvent()
		
		Select gEvent
			Case #PB_Event_CloseWindow
				gQuit= #True
				
			Case #PB_Event_Gadget
				Select EventGadget()
					Case #But1
						If Not notify_is_initted() : notify_init("Test-Application") : EndIf
						gNotify = notify_notification_new("Application-Info", "Desktop notification with 'libnotify'!" + #LF$ + "2. line - autohide", "document-save")
						notify_notification_set_timeout(gNotify, 5000);                 5 seconds
						; set low urgency (or deactivate it) ...
						notify_notification_set_urgency(gNotify, #NOTIFY_URGENCY_LOW);  autohide
						notify_notification_show(gNotify)
						g_object_unref_(gNotify)
					Case #But2
						If Not notify_is_initted() : notify_init("Test-Application") : EndIf
						gNotify = notify_notification_new("Application-Info", "Desktop notification with 'libnotify'!" + #LF$ + "Urgent mode, needs click!", "dialog-warning")
						; set high urgency ...
						notify_notification_set_urgency(gNotify, #NOTIFY_URGENCY_CRITICAL); to click
						notify_notification_show(gNotify)
						g_object_unref_(gNotify)
				EndSelect
				
		EndSelect
		
	Until gQuit
EndIf

If notify_is_initted() : notify_uninit() : EndIf
; IDE Options = PureBasic 5.40 LTS Beta 9 (Linux - x86)
; CursorPosition = 111
; FirstLine = 78
; Folding = -
; EnableUnicode
; EnableXP