﻿; Module/File:     Directory_SetMonitorSimple.pb
; Function:        Adds a gLib-Monitor for a directory - Linux
; Author:          Omi
; Date:            Jan. 26, 2016
; Version:         0.1
; Target Compiler: PureBasic 5.22/5.31
; Target OS:       Linux: (X/K/L)ubuntu, Mint, 32/64, Ascii/Uni
;--------------------------------------------------------------

; GFileMonitorFlags
#G_FILE_MONITOR_NONE             = 0
#G_FILE_MONITOR_WATCH_MOUNTS     = 1
#G_FILE_MONITOR_SEND_MOVED       = 2
;#G_FILE_MONITOR_WATCH_HARD_LINKS = 4

; Structure; GFileMonitorEvent
; 	#G_FILE_MONITOR_EVENT_CHANGED;           = 0
; 	#G_FILE_MONITOR_EVENT_CHANGES_DONE_HINT; = 1
; 	#G_FILE_MONITOR_EVENT_DELETED;           = 2
; 	#G_FILE_MONITOR_EVENT_CREATED;           = 3
; 	#G_FILE_MONITOR_EVENT_ATTRIBUTE_CHANGED; = 4
; 	#G_FILE_MONITOR_EVENT_PRE_UNMOUNT;       = 5
; 	#G_FILE_MONITOR_EVENT_UNMOUNTED;         = 6
; 	#G_FILE_MONITOR_EVENT_MOVED;             = 7
; EndStructure

; see also ...
;		g_file_monitor_cancel(GFileMonitor *monitor)
;		g_file_monitor_is_cancelled(GFileMonitor *monitor)
;		g_file_monitor_set_rate_limit(GFileMonitor *monitor, gint limit_msecs)

EnableExplicit

ImportC ""
	g_file_monitor_directory(*file, flags, *cancellable, *err)
	g_file_new_for_path(path.p-utf8)
	g_file_get_basename(*file)
	g_file_get_path(*file)
	g_file_monitor_cancel(*monitor)
	g_signal_connect(*instance, detailed_signal.p-ascii, *c_handler, *pdata, destroy= 0, flags= 0) As "g_signal_connect_data"
EndImport

; Object constants
#MainWin  = 0

#Txt1     = 0
#Txt2     = 1

#WinX=300
#WinY=200
#WinW=400
#WinH=200

Global.i gEvent, gQuit
Global *gErr.GError
Global *gFMonitor
Global.s gFolder= GetTemporaryDirectory()


ProcedureC Callback_FileChanged(*fMonitor, *file, *sfile, EventType, user_data)
	If *file
		;Debug PeekS(g_file_get_basename(*file), -1, #PB_UTF8);                      filename
		Debug PeekS(g_file_get_path(*file), -1, #PB_UTF8) + " : " + Str(EventType);  filename with path
	EndIf
EndProcedure

Procedure DirMonitor_Add(Folder.s, *Callback)
	Protected *folder  = g_file_new_for_path(Folder)
	Protected *fMonitor= g_file_monitor_directory(*folder, #G_FILE_MONITOR_WATCH_MOUNTS | #G_FILE_MONITOR_SEND_MOVED, 0, @*gErr)
	g_signal_connect(*fMonitor, "changed", *Callback, 0)
	ProcedureReturn *fMonitor
EndProcedure

Procedure Monitor_Cancel(*fMonitor)
	g_file_monitor_cancel(*gFMonitor)
	g_object_unref_(*gFMonitor)
EndProcedure


If OpenWindow(#MainWin, #WinX, #WinY, #WinW, #WinH, "Simple gFile Directory-Monitor", #PB_Window_SystemMenu | #PB_Window_ScreenCentered)
	TextGadget(#Txt1, 5, 5, 390, 20, "Create or copy file in '" + gFolder + "'. Change and delete it!")
	TextGadget(#Txt2, 5, 30, 390, 20, "And watch Debug!")
	
	*gErr= 0
	*gFMonitor= DirMonitor_Add(gFolder, @Callback_FileChanged())
	
	Repeat
		gEvent= WaitWindowEvent()
		
		Select gEvent
			Case #PB_Event_CloseWindow
				gQuit= #True

		EndSelect
		
	Until gQuit
EndIf

Monitor_Cancel(*gFMonitor)
; IDE Options = PureBasic 5.40 LTS Beta 9 (Linux - x86)
; CursorPosition = 72
; FirstLine = 53
; Folding = -
; EnableUnicode
; EnableXP