﻿; Module/File:     Printer_CupsAPI_Demo.pb
; Function:        Demo: List cups printers, get/set infos, server and user-default-printer  - Linux
; Author:          Omi
; Date:            Mar. 16, 2016
; Version:         0.1
; Target Compiler: PureBasic 5.22/5.31/5.40
; Target OS:       Linux: (X/K/L)ubuntu, Mint, 32/64, Ascii/Uni
;--------------------------------------------------------------
; need libcups2dev
; install e.g. : sudo apt-get install --assume-yes libcups2 libcups2-dev
; like cups pdf: sudo apt-get install cups-pdf
;               + sudo /etc/init.d/cups restart

EnableExplicit

ImportC "-lcups"
	cupsGetDefault()
	cupsUser()
	cupsUserAgent()
	cupsServer()
	cupsGetPPD(name.p-utf8)
	cupsGetDests(*dests)
; 	cupsGetDest(*name, *instance, num_dests.l, *dests);                                if name = #NULL
; 	cupsGetDestSName(name.p-utf8, *instance, num_dests.l, *dests) As "cupsGetDest";    if name <> #NULL
	cupsSetDefaultDest(name.p-utf8, *instance, num_dests.l, *dests);                   if instance = #NULL
; 	cupsSetDefaultDest(name.p-utf8, *instance.p-utf8, num_dests.l, *dests);            if instance <> #NULL
	cupsSetDests(num_dests.l, *dests)
	cupsFreeDests(num_dests.l, *dests)
EndImport

Structure cups_option_s
	*name
	*value
EndStructure

Structure cups_dest_s
	*name
	*instance
	is_default.l
	num_options.l
	*options.cups_option_s
EndStructure


; Object constants
#MainWin  = 0

Enumeration
	#CupsUser
	#CupsUserAgent
	#CupsServer
	#CupsServerPrinterDef
	#CupsNPrinter
	#CupsListPrinters
	#CupsListOptions
	#CupsSetUserDef
	#CupsSetServerDef
EndEnumeration

Global.i gEvent, gQuit
Global.i gI
Global   *gDefPrtDest.cups_dest_s
Global   *gDefPrtDests.cups_dest_s
Global.l gNumDests

Procedure.s Cups_GetUser()
	Protected   *CupsUser= cupsUser()
	Protected.s S= ""
	
	If *CupsUser
		S= PeekS(*CupsUser, -1, #PB_UTF8)
	EndIf
	ProcedureReturn S
EndProcedure

Procedure.s Cups_GetUserAgent()
	Protected   *CupsUserAgent= cupsUserAgent()
	Protected.s S= ""
	
	If *CupsUserAgent
		S= PeekS(*CupsUserAgent, -1, #PB_UTF8)
	EndIf
	ProcedureReturn S
EndProcedure

Procedure.s Cups_GetServer()
	Protected   *CupsServer= cupsServer()
	Protected.s S= ""
	
	If *CupsServer
		S= PeekS(*CupsServer, -1, #PB_UTF8)
	EndIf
	ProcedureReturn S
EndProcedure

Procedure.s Cups_GetDefaultServerPrinter();                          Only w. Cups API ?
	Protected   *gDefSrvPrt= cupsGetDefault()
	Protected.s S= ""
	
	If *gDefSrvPrt
		S= PeekS(*gDefSrvPrt, -1, #PB_UTF8)
	EndIf
	ProcedureReturn S
EndProcedure

Procedure.l Cups_GetDestinations()
	gNumDests= cupsGetDests(@*gDefPrtDests)
EndProcedure

Procedure Cups_GetPrinterOptions(PrinterId.i)
	Protected   *PrtDest.cups_dest_s= *gDefPrtDests+ (PrinterId * SizeOf(cups_dest_s))
	Protected   *PrinterOption.cups_option_s
	Protected.i I
	
	If *PrtDest\options
		ClearGadgetItems(#CupsListOptions)
		*PrinterOption= *PrtDest\options
		For I= 1 To *PrtDest\num_options
			AddGadgetItem(#CupsListOptions, I - 1, RSet(Str(I), 3) + "." + #LF$ + 
			                                       PeekS(*PrinterOption\name, -1, #PB_UTF8) + #LF$ + 
			                                       PeekS(*PrinterOption\value, -1, #PB_UTF8))
			*PrinterOption+ SizeOf(cups_option_s);                         step through struct array
		Next I
	EndIf
EndProcedure

Procedure Cups_SetUserDefPrinter(PrinterId.i)
	Protected   *PrtDest.cups_dest_s
	Protected.i I
	
	For I= 0 To gNumDests- 1
		*PrtDest= *gDefPrtDests+ (I * SizeOf(cups_dest_s));              step through struct array
		If I = PrinterId
			*PrtDest\is_default= #True;                                    sets user default printer
		Else
			*PrtDest\is_default= #False
		EndIf
		SetGadgetItemText(#CupsListPrinters, I, Str(*PrtDest\is_default), 2)
	Next I
EndProcedure

Procedure Cups_SetServerDefPrinter(PrinterId.i)
	Protected   *PrtDest.cups_dest_s= *gDefPrtDests+ (PrinterId * SizeOf(cups_dest_s))
	Protected.s sName
	
	If *PrtDest
		If *PrtDest\name
			sName= PeekS(*PrtDest\name, -1, #PB_UTF8)
			If sName <> ""
				RunProgram("lpadmin", "-d " + sName, "");                    set server default printer
; 				cupsSetDefaultDest(sName, #Null, gNumDests, *gDefPrtDests);  doesn't do the job ???
			EndIf
		EndIf
	EndIf
	Delay(200)
EndProcedure


If OpenWindow(#MainWin, 300, 200, 800, 395, "Cups-API: infos + get/set default printer", #PB_Window_SystemMenu | #PB_Window_ScreenCentered)
	TextGadget(#PB_Any,                   5,  10, 170,  22, "Cups user: ", #PB_Text_Right)
	TextGadget(#PB_Any,                   5,  40, 170,  22, "Cups user agent: ", #PB_Text_Right)
	TextGadget(#PB_Any,                   5,  70, 170,  22, "Cups server: ", #PB_Text_Right)
	TextGadget(#PB_Any,                   5, 100, 170,  22, "Default server printer: ", #PB_Text_Right)
	TextGadget(#CupsNPrinter,             5, 130, 170,  22, "Listed Cups printers: ")
	TextGadget(#PB_Any,                 360, 130, 170,  22, "Options for selected printer: ")
	
	StringGadget(#CupsUser,             180,   5, 400,  26, Cups_GetUser(), #PB_String_ReadOnly)
	StringGadget(#CupsUserAgent,        180,  35, 400,  26, Cups_GetUserAgent(), #PB_String_ReadOnly)
	StringGadget(#CupsServer   ,        180,  65, 400,  26, Cups_GetServer(), #PB_String_ReadOnly)
	StringGadget(#CupsServerPrinterDef, 180,  95, 400,  26, Cups_GetDefaultServerPrinter(), #PB_String_ReadOnly)
	Cups_GetDestinations()
	SetGadgetText(#CupsNPrinter, "Listed Cups printers: "+ Str(gNumDests))
	
	ListIconGadget(#CupsListPrinters,     5, 150, 350, 170, "Printer name", 150, #PB_ListIcon_GridLines)
	AddGadgetColumn(#CupsListPrinters,    1, "Instance name", 100)
	AddGadgetColumn(#CupsListPrinters,    2, "User-Default", 70)
	gtk_tree_selection_set_mode_(gtk_tree_view_get_selection_(GadgetID(#CupsListPrinters)), #GTK_SELECTION_BROWSE)
	
	ListIconGadget(#CupsListOptions,    360, 150, 435, 238, "#", 35, #PB_ListIcon_GridLines)
	AddGadgetColumn(#CupsListOptions,     1, "Option",  160)
	AddGadgetColumn(#CupsListOptions,     2, "Value",   250)
	gtk_tree_selection_set_mode_(gtk_tree_view_get_selection_(GadgetID(#CupsListOptions)), #GTK_SELECTION_NONE)
	
	ButtonGadget(#CupsSetUserDef,         5, 327, 350,  28, "Selected printer as user default")
	ButtonGadget(#CupsSetServerDef,       5, 360, 350,  28, "Selected printer as server default")
	
	If gNumDests
		*gDefPrtDest= *gDefPrtDests
		For gI= 0 To gNumDests- 1
			AddGadgetItem(#CupsListPrinters, gI, "")
			
			If *gDefPrtDest\name
				SetGadgetItemText(#CupsListPrinters, gI, PeekS(*gDefPrtDest\name, -1, #PB_UTF8), 0)
			EndIf
			
			If *gDefPrtDest\instance
				SetGadgetItemText(#CupsListPrinters, gI, PeekS(*gDefPrtDest\instance, -1, #PB_UTF8), 1)
			EndIf
			
			SetGadgetItemText(#CupsListPrinters, gI, Str(*gDefPrtDest\is_default), 2)
			
			*gDefPrtDest+ SizeOf(cups_dest_s);                             step through struct array
		Next gI
	EndIf
	
	If cupsGetPPD(GetGadgetItemText(#CupsListPrinters, 0, 0))
		Debug "PPD-file (PostscriptPrinterDescription): " + PeekS(cupsGetPPD(GetGadgetItemText(#CupsListPrinters, 0, 0)), -1, #PB_UTF8)
	EndIf
	
	
	Repeat
		gEvent= WaitWindowEvent()
		
		Select gEvent
				
			Case #PB_Event_CloseWindow
				gQuit= #True
				
			Case #PB_Event_Gadget
				
				Select EventGadget()
					Case #CupsListPrinters
						If EventType() = #PB_EventType_Change
							If GetGadgetState(#CupsListPrinters) > -1
								Cups_GetPrinterOptions(GetGadgetState(#CupsListPrinters))
							EndIf
						EndIf
					
					Case #CupsSetUserDef
						If GetGadgetState(#CupsListPrinters) > -1
							Cups_SetUserDefPrinter(GetGadgetState(#CupsListPrinters))
							cupsSetDests(gNumDests, *gDefPrtDests);             can't be called within a Proc
						EndIf
					
					Case #CupsSetServerDef
						If GetGadgetState(#CupsListPrinters) > -1
							Cups_SetServerDefPrinter(GetGadgetState(#CupsListPrinters))
							SetGadgetText(#CupsServerPrinterDef, Cups_GetDefaultServerPrinter())
						EndIf
						
				EndSelect
				
		EndSelect
		
	Until gQuit
EndIf

If gNumDests
	cupsFreeDests(gNumDests, *gDefPrtDests)
EndIf
; IDE Options = PureBasic 5.42 LTS (Linux - x86)
; CursorPosition = 227
; FirstLine = 221
; Folding = --
; EnableUnicode
; EnableXP