﻿; Module/File:     Window_SetCursor.pb
; Function:        Sets cursor for window - Linux gtk2/gtk3
; Author:          Shardik/Omi
; Date:            Sep. 18, 2014
; Version:         0.2 updt. gtk3
; Target Compiler: PureBasic 5.22/5.31/5.40
; Target OS:       Linux: (X/K/L)ubuntu, Mint, 32/64, Ascii/Uni
;--------------------------------------------------------------

; Enumeration GdkCursorType
; 	#GDK_X_CURSOR           = 0
; 	#GDK_ARROW              = 2
; 	#GDK_BASED_ARROW_DOWN   = 4
; 	#GDK_BASED_ARROW_UP     = 6
; 	#GDK_BOAT               = 8
; 	#GDK_BOGOSITY           = 10
; 	#GDK_BOTTOM_LEFT_CORNER = 12;   (resize southwest corner)
; 	#GDK_BOTTOM_RIGHT_CORNER= 14;   (resize southeast corner)
; 	#GDK_BOTTOM_SIDE        = 16;   (resize bottom side)
; 	#GDK_BOTTOM_TEE         = 18
; 	#GDK_BOX_SPIRAL         = 20
; 	#GDK_CENTER_PTR         = 22
; 	#GDK_CIRCLE             = 24
; 	#GDK_CLOCK              = 26
; 	#GDK_COFFEE_MUG         = 28
; 	#GDK_CROSS              = 30
; 	#GDK_CROSS_REVERSE      = 32
; 	#GDK_CROSSHAIR          = 34;   (crosshair)
; 	#GDK_DIAMOND_CROSS      = 36
; 	#GDK_DOT                = 38
; 	#GDK_DOTBOX             = 40
; 	#GDK_DOUBLE_ARROW       = 42
; 	#GDK_DRAFT_LARGE        = 44
; 	#GDK_DRAFT_SMALL        = 46
; 	#GDK_DRAPED_BOX         = 48
; 	#GDK_EXCHANGE           = 50
; 	#GDK_FLEUR              = 52;   (for moving objects)
; 	#GDK_GOBBLER            = 54
; 	#GDK_GUMBY              = 56
; 	#GDK_HAND1              = 58;   (a right-pointing hand)
; 	#GDK_HAND2              = 60;   (a left-pointing hand)
; 	#GDK_HEART              = 62
; 	#GDK_ICON               = 64
; 	#GDK_IRON_CROSS         = 66
; 	#GDK_LEFT_PTR           = 68
; 	#GDK_LEFT_SIDE          = 70;   (resize left side)
; 	#GDK_LEFT_TEE           = 72
; 	#GDK_LEFTBUTTON         = 74
; 	#GDK_LL_ANGLE           = 76
; 	#GDK_LR_ANGLE           = 78
; 	#GDK_MAN                = 80
; 	#GDK_MIDDLEBUTTON       = 82
; 	#GDK_MOUSE              = 84
; 	#GDK_PENCIL             = 86
; 	#GDK_PIRATE             = 88
; 	#GDK_PLUS               = 90
; 	#GDK_QUESTION_ARROW     = 92
; 	#GDK_RIGHT_PTR          = 94;   (right-facing arrow)
; 	#GDK_RIGHT_SIDE         = 96;   (resize right side)
; 	#GDK_RIGHT_TEE          = 98
; 	#GDK_RIGHTBUTTON        = 100
; 	#GDK_RTL_LOGO           = 102
; 	#GDK_SAILBOAT           = 104
; 	#GDK_SB_DOWN_ARROW      = 106
; 	#GDK_SB_H_DOUBLE_ARROW  = 108;  (move vertical splitter)
; 	#GDK_SB_LEFT_ARROW      = 110
; 	#GDK_SB_RIGHT_ARROW     = 112
; 	#GDK_SB_UP_ARROW        = 114
; 	#GDK_SB_V_DOUBLE_ARROW  = 116;  (move horizontal splitter)
; 	#GDK_SHUTTLE            = 118
; 	#GDK_SIZING             = 120
; 	#GDK_SPIDER             = 122
; 	#GDK_SPRAYCAN           = 124
; 	#GDK_STAR               = 126
; 	#GDK_TARGET             = 128
; 	#GDK_TCROSS             = 130
; 	#GDK_TOP_LEFT_ARROW     = 132
; 	#GDK_TOP_LEFT_CORNER    = 134;  (resize northwest corner)
; 	#GDK_TOP_RIGHT_CORNER   = 136;  (resize northeast corner)
; 	#GDK_TOP_SIDE           = 138;  (resize top side)
; 	#GDK_TOP_TEE            = 140
; 	#GDK_TREK               = 142
; 	#GDK_UL_ANGLE           = 144
; 	#GDK_UMBRELLA           = 146
; 	#GDK_UR_ANGLE           = 148
; 	#GDK_WATCH              = 150;  (busy)
; 	#GDK_XTERM              = 152;  (I-beam)
; 	#GDK_LAST_CURSOR        = 153
	#GDK_BLANK_CURSOR         = -2;   (Blank cursor). Since 2.16
; 	#GDK_CURSOR_IS_PIXMAP   = -1
; EndEnumeration


EnableExplicit

ImportC ""
	gtk_widget_get_window(*widget.GtkWidget)
EndImport

; Object constants
#MainWin  = 0

#Button1 = 0
#Button2 = 1
#Button3 = 2
#Button4 = 3
#Button5 = 4

#WinX=100
#WinY=100
#WinW=170
#WinH=160

Global.i gEvent, gQuit
Global *Cursor.GdkCursor

OpenWindow(#MainWin, 100, 100, 170, 160, "Cursor-Test")
ButtonGadget(#Button1, 10, 10, WindowWidth(0) - 20, 24, "X-Cursor")
ButtonGadget(#Button2, 10, 40, WindowWidth(0) - 20, 24, "O-Cursor")
ButtonGadget(#Button3, 10, 70, WindowWidth(0) - 20, 24, "+-Cursor")
ButtonGadget(#Button4, 10, 100, WindowWidth(0) - 20, 24, "Sprühflasche")
ButtonGadget(#Button5, 10, 130, WindowWidth(0) - 20, 24, "_blank_")

Procedure ChangeCursor(Window, CursorID.i)
	*Cursor= gdk_cursor_new_(CursorID)
	If *Cursor
		gdk_window_set_cursor_(gtk_widget_get_window(WindowID(Window)), *Cursor)
	EndIf
EndProcedure

Repeat
	gEvent= WaitWindowEvent()
	Select gEvent
    Case #PB_Event_CloseWindow
      gQuit= #True
    Case #PB_Event_Gadget
      Select EventGadget()
        Case #Button1
          ChangeCursor(#MainWin, #GDK_X_CURSOR)
        Case #Button2
          ChangeCursor(#MainWin, #GDK_CIRCLE)
        Case #Button3
          ChangeCursor(#MainWin, #GDK_TCROSS)
        Case #Button4
          ChangeCursor(#MainWin, #GDK_SPRAYCAN)
        Case #Button5
          ChangeCursor(#MainWin, -2)
      EndSelect
  EndSelect
Until gQuit
; IDE Options = PureBasic 5.42 LTS (Linux - x64)
; CursorPosition = 122
; FirstLine = 108
; Folding = -
; EnableUnicode
; EnableXP